#include <stdio.h>
#include <memory.h>
#include <stdlib.h>
#include "mpeg.h"

//if you want to run TextMode please #define TextMode
//#define TextMode

//default intra quantization table
int default_i_qTab[64]={
    8,  16, 19, 22, 26, 27, 29, 34,
    16, 16, 22, 24, 27, 29, 34, 37,
    19, 22, 26, 27, 29, 34, 34, 38,
    22, 22, 26, 27, 29, 34, 37, 40,
    22, 26, 27, 29, 32, 35, 40, 48,
    26, 27, 29, 32, 35, 40, 48, 58,
    26, 27, 29, 34, 38, 46, 56, 69,
    27, 29, 35, 38, 46, 56, 69, 83
};

//mapping from sequence order to zig-zag ordera
int seq_to_zz[64]={
    0,  1,  5,  6,  14, 15, 27, 28,
    2,  4,  7,  13, 16, 26, 29, 42,
    3,  8,  12, 17, 25, 30, 41, 43,
    9,  11, 18, 24, 31, 40, 44, 53,
    10, 19, 23, 38, 46, 51, 55, 60,
    21, 34, 37, 47, 50, 56, 59, 61,
    35, 36, 48, 49, 57, 58, 62, 63
};

//Permutation for DCT Coefficients
//  It is necessary to my ScaleIDCT function.
const int IDCT_P[64] =
{
	   0,   5,  40,  16,  45,   2,   7,  42,
	  21,  56,   8,  61,  18,  47,   1,   4,
	  41,  23,  58,  13,  32,  24,  37,  10,
	  63,  17,  44,   3,   6,  43,  20,  57,
	  15,  34,  29,  48,  53,  26,  39,   9,
	  60,  19,  46,  22,  59,  12,  33,  31,
	  50,  55,  25,  36,  11,  62,  14,  35,
	  28,  49,  52,  27,  38,  30,  51,  54	
};


inline int Sign(int a){
    if(a==0) return 0;
    if(a>0) return 1;
    return -1;
}

//Each quantization Table should be enhanced
// to fit my ScaleIDCT function
void EnhanceQuantizationTable(int *qtab)
{
	static int table[] = {
		0,	1,  5,	6,	14,	15,	27,	28, 
		2,	4,	7,	13,	16,	26,	29,	42,
		3,	8,	12,	17,	25,	30,	41,	43,
		9,	11,	18,	24,	31,	40,	44,	53,
		10, 19, 23, 32, 39, 45, 52, 54,
		20, 22, 33, 38, 46, 51, 55, 60,
		21, 34, 37, 47, 50, 56, 59, 61,
		35, 36, 48, 49, 57, 58, 62, 63
	};
	int i ;
	for( i = 0 ; i < 8 ; i ++ )
	{
		qtab[table[0*8+i]] *= 90  ;
		qtab[table[4*8+i]] *= 90  ;
		qtab[table[2*8+i]] *= 118 ;
		qtab[table[6*8+i]] *= 49  ;
		qtab[table[5*8+i]] *= 71  ;
		qtab[table[1*8+i]] *= 126 ;
		qtab[table[7*8+i]] *= 25  ;
		qtab[table[3*8+i]] *= 106 ;
	}
	for( i = 0 ; i < 8 ; i ++ )
	{
		qtab[table[0+8*i]] *= 90  ;
		qtab[table[4+8*i]] *= 90  ;
		qtab[table[2+8*i]] *= 118 ;
		qtab[table[6+8*i]] *= 49  ;
		qtab[table[5+8*i]] *= 71  ;
		qtab[table[1+8*i]] *= 126 ;
		qtab[table[7+8*i]] *= 25  ;
		qtab[table[3+8*i]] *= 106 ;
	}
	for( i = 0 ; i < 64 ; i++ ) {
		qtab[i] >>= 6 ;
	}
}

//ScaleIDCT():
//  input: IDCT_Source
//  output: matrix
void MpegClass::ScaleIDCT(int* matrix)
{
	static int p[8][8] ;
	int t0 , t1 , t2 , t3, i ;
	int src0, src1, src2, src3, src4, src5, src6, src7;
	int det0, det1, det2, det3, det4, det5, det6, det7;

	for( i = 0 ; i < 8 ; i++ )
	{
		src0 = IDCT_Source[0*8+i] ;
		src1 = IDCT_Source[1*8+i] ;
		src2 = IDCT_Source[2*8+i] - IDCT_Source[3*8+i] ;
		src3 = IDCT_Source[3*8+i] + IDCT_Source[2*8+i] ;
		src4 = IDCT_Source[4*8+i] - IDCT_Source[7*8+i] ;
		src6 = IDCT_Source[5*8+i] - IDCT_Source[6*8+i] ;
		t0   = IDCT_Source[5*8+i] + IDCT_Source[6*8+i] ;
		t1   = IDCT_Source[4*8+i] + IDCT_Source[7*8+i] ;
		src5 = t0 - t1 ;
		src7 = t0 + t1 ;
		//
		det4 =-src4 * 480 - src6 * 192 ;
		det5 = src5 * 384 ;
		det6 = src6 * 480 - src4 * 192 ;
		det7 = src7 * 256 ;
		t0   = src0 * 256 ;
		t1   = src1 * 256 ;
		t2   = src2 * 384 ;
		t3   = src3 * 256 ;
		det3 = t3      ;
		det0 = t0 + t1 ;
		det1 = t0 - t1 ;
		det2 = t2 - t3 ;
		//
		src0 = det0 + det3 ;
		src1 = det1 + det2 ;
		src2 = det1 - det2 ;
		src3 = det0 - det3 ;
		src4 = det6 - det4 - det5  - det7 ;
		src5 = det5 - det6 + det7 ;
		src6 = det6 - det7 ;
		src7 = det7 ;
		//
		p[0][i] = ( src0 + src7 +(1<<12))>>13 ;
		p[1][i] = ( src1 + src6 +(1<<12))>>13 ;
		p[2][i] = ( src2 + src5 +(1<<12))>>13 ;
		p[3][i] = ( src3 + src4 +(1<<12))>>13 ;
		p[4][i] = ( src3 - src4 +(1<<12))>>13 ;
		p[5][i] = ( src2 - src5 +(1<<12))>>13 ;
		p[6][i] = ( src1 - src6 +(1<<12))>>13 ;
		p[7][i] = ( src0 - src7 +(1<<12))>>13 ;
	}
	//
	for( i = 0 ; i < 8 ; i++ )
	{
		src0 = p[i][0] ;
		src1 = p[i][1] ;
		src2 = p[i][2] - p[i][3] ;
		src3 = p[i][3] + p[i][2] ;
		src4 = p[i][4] - p[i][7] ;
		src6 = p[i][5] - p[i][6] ;
		t0   = p[i][5] + p[i][6] ;
		t1   = p[i][4] + p[i][7] ;
		src5 = t0 - t1 ;
		src7 = t0 + t1 ;
		//
		det4 =-src4 * 480 - src6 * 192 ;
		det5 = src5 * 384 ;
		det6 = src6 * 480 - src4 * 192 ;
		det7 = src7 * 256 ;
		t0   = src0 * 256 ;
		t1   = src1 * 256 ;
		t2   = src2 * 384 ;
		t3   = src3 * 256 ;
		det3 = t3      ;
		det0 = t0 + t1 ;
		det1 = t0 - t1 ;
		det2 = t2 - t3 ;
		//
		src0 = det0 + det3 ;
		src1 = det1 + det2 ;
		src2 = det1 - det2 ;
		src3 = det0 - det3 ;
		src4 = det6 - det4 - det5  - det7 ;
		src5 = det5 - det6 + det7 ;
		src6 = det6 - det7 ;
		src7 = det7 ;
		//
		matrix[0] = ( src0 + src7 +(1<<12))>>13 ;
		matrix[1] = ( src1 + src6 +(1<<12))>>13 ;
		matrix[2] = ( src2 + src5 +(1<<12))>>13 ;
		matrix[3] = ( src3 + src4 +(1<<12))>>13 ;
		matrix[4] = ( src3 - src4 +(1<<12))>>13 ;
		matrix[5] = ( src2 - src5 +(1<<12))>>13 ;
		matrix[6] = ( src1 - src6 +(1<<12))>>13 ;
		matrix[7] = ( src0 - src7 +(1<<12))>>13 ;
		matrix+=8;
	}
}	


MpegClass::MpegClass(FILE* in){
    int i;
    input=new bi_io_class(in);
    IDCT_Source = new int[64];
    past_picture=0;
    current_picture=1;
    future_picture=2;
    err=stdout;

    //read the first picture and initial picture[3].
    input->next_start_code();
    if(input->getn(32) != 0x000001B3){
        fprintf(err,"ERROR: this file is not Mpeg sequence! (1)\n");
        exit(0);
    }
    get_sequence_header();

    for(i=0; i<3; i++) picture[i]=new int[v_size*h_size];

    //group of pictures
    if(input->getn(32) != 0x000001B8){
        fprintf(err,"ERROR: this file is not Mpeg sequence! (2)\n");
        exit(0);
    }
    time_code=input->getn(25);
    closed_gop=(int)input->getn(1);
    broken_link=(int)input->getn(1);
    get_extension_user_data();
    temporal_ref=0;
    picture_count=0;

    #ifdef TextMode
        printf("================== Group of pictures =============\n");
        printf("time_code=%X\n", time_code);
        printf("closed_gop=%i\n", closed_gop);
        printf("broken_link=%i\n", broken_link);
    #endif

    //the 1st picture
    if(input->getn(32) != 0x00000100){
        fprintf(err,"ERROR: this file is not Mpeg sequence! (3)\n");
        exit(0);
    }
    picture_count++;
    temporal_ref=(int)input->getn(10);
    picture_coding_type=(int)input->getn(3);
    vbv_delay=(int)input->getn(16);

    #ifdef TextMode
        printf("--------------PICTURE %i-------------\n", picture_count);
        printf("temporal_ref=%i\n", temporal_ref);
        printf("picture_coding_type=%i\n", picture_coding_type);
        printf("vbv_delay=%X\n", vbv_delay);
    #endif
    
    if(picture_coding_type==2||picture_coding_type==3){
        fprintf(err,"ERROR: this file is not Mpeg sequence! (4)\n");
        exit(0);
    }
    while(input->getn(1))
        input->getn(8);
    get_extension_user_data();

    seq_num[future_picture]=0;
    pel=picture[future_picture];//decode for future picture

    #ifdef TextMode
        printf("current_picture=%i\n", current_picture);
        printf("future_picture=%i\n", future_picture);
        printf("past_picture=%i\n", past_picture);
    #endif

    do{
       slice();
    } while(input->next32()>=0x00000101 && 
            input->next32()<=0x000001AF);

}


//It is sure to reach a start code
void MpegClass::get_extension_user_data(void){
    input->next_start_code();
    if(input->next32()==extension_start_code){
        input->getn(32);
        while(input->next24()!=0x00000001)
            input->getn(8);
        #ifdef TextMode
            printf("Extesion data!\n");
        #endif
    }
    if(input->next32()==user_data_start_code){
        input->getn(32);
        while(input->next24()!=0x00000001)
            input->getn(8);
        #ifdef TextMode
            printf("User data!\n");
        #endif
    }

}

// start_code has been used.
void MpegClass::get_sequence_header(void){
    static int to_picture_rate[9]={
        0, 24, 24, 25, 30, 30, 50, 60, 60
    };
    int i;
    h_size=(int)input->getn(12);
    v_size=(int)input->getn(12);
    pel_aspect_ratio=(int)input->getn(4);
    picture_rate=to_picture_rate[input->getn(4)];
    bit_rate=(int)input->getn(18);
    if(input->getn(1)==0){
        fprintf(err,"ERROR: marker_bit=0\n");
        //input->next_start_code();
        //goto EndOfSlice;
    }
    vbv_buffer_size=input->getn(10);
    constrained_parameter_flag=input->getn(1);
    if(input->getn(1)){// load_intra_quantizer_matrix
        for(i=0; i<64; i++) 
            i_qTab[i]=(int)input->getn(8);
    }else{
        for(i=0; i<64; i++)
            i_qTab[seq_to_zz[i]]=default_i_qTab[i];
    }
    if(input->getn(1)){// load_non_intra_quantizer_matrix
        for(i=0; i<64; i++)
            n_qTab[i]=(int)input->getn(8);
    }else{
        for(i=0; i<64; i++)
            n_qTab[i]=16;
    }
    i_qTab[0]=1;
    EnhanceQuantizationTable(i_qTab); //because of Scaled IDCT
    EnhanceQuantizationTable(n_qTab); //because of Scaled IDCT

    get_extension_user_data();

    //The following is to get some variable
    mb_width=(h_size+15)/16;
    mb_height=(v_size+15)/16;

    #ifdef TextMode
        printf("-------------- Sequence Header -------------------\n");
        printf("h_size=%i    v_size=%i\n", h_size, v_size);
        printf("pel_aspect_ratio=%i", pel_aspect_ratio);
        printf("picture_rate=%i\n", picture_rate);
        printf("bit_rate=%i\n", bit_rate);
        printf("vbv_buffer_size=%i\n", vbv_buffer_size);
        printf("constrained_parameter_flag=%i\n", constrained_parameter_flag);
        printf("mb_width=%i     mb_height=%i\n", mb_width,mb_height);
        printf("i_qTab=\n");
        for(i=0; i<64; i++){
            printf("%4i ", i_qTab[i]);
            if( (i%8) == 7 ) printf("\n");
        }
        printf("n_qTab=\n");
        for(i=0; i<64; i++){
            printf("%4i ", n_qTab[i]);
            if( (i%8) == 7 ) printf("\n");
        }
        printf("--------------------------------------------------\n");
    #endif
}


/*decode():
    When you call decode(), the function will decode one picture.
    If the picture is B-picture, picture[current_picture] is that.
    If the picture is I or P-picture, picture[future_picture] is that.

    The program will ensure that picture[current_picture] is the
        picture to be displayed.

    Next time you call decode(), the function will decode next picture.
*/
int MpegClass::decode(void){
    input->next_start_code();
    switch(input->next32()){

    case sequence_header_code :

        input->getn(32);
        get_sequence_header();

    case group_start_code:

        input->getn(32);
        time_code=input->getn(25);
        closed_gop=(int)input->getn(1);
        broken_link=(int)input->getn(1);
        get_extension_user_data();
        temporal_ref=0;
        picture_count=0;

        #ifdef TextMode
            printf("================== Group of pictures ============\n");
            printf("time_code=%X\n", time_code);
            printf("closed_gop=%i\n", closed_gop);
            printf("broken_link=%i\n", broken_link);
        #endif

    case picture_start_code:

        input->getn(32);
        picture_count++;
        temporal_ref=(int)input->getn(10);
        picture_coding_type=(int)input->getn(3);
        vbv_delay=(int)input->getn(16);
        if(picture_coding_type==2||picture_coding_type==3){
            full_pel_forward_vector=(int)input->getn(1);
            forward_f_code=(int)input->getn(3);
            forward_r_size=forward_f_code-1;
            forward_f= 1 << forward_r_size;
        }
        if(picture_coding_type==3){
            full_pel_backward_vector=(int)input->getn(1);
            backward_f_code=(int)input->getn(3);
            backward_r_size=backward_f_code-1;
            backward_f= 1 << backward_r_size;
        }
        while(input->getn(1))
            input->getn(8);
        get_extension_user_data();

        //Handle the pictures.....
        // make pel[] is the destination of current decoding
        //    pel_past[], pel_future are reference picture.
        if(picture_coding_type==1 || picture_coding_type==2){
            
            prev_output_seq=seq_num[current_picture];
            current_picture=future_picture;
            future_picture=past_picture;
            past_picture=current_picture;
            seq_num[future_picture]=temporal_ref;
            pel=picture[future_picture];//decode for future picture
            pel_past=picture[past_picture];

        }else if(picture_coding_type==3){

            prev_output_seq=seq_num[current_picture];
            switch(past_picture+future_picture){
            case 1: current_picture=2; break;
            case 2: current_picture=1; break;
            case 3: current_picture=0; break;
            default: fprintf(err,"ERROR: past_picture=%i, future_picture=%i\n",
                                past_picture, future_picture);
            }
            seq_num[current_picture]=temporal_ref;
            pel=picture[current_picture];//decode for current picture
            pel_past=picture[past_picture];
            pel_future=picture[future_picture];
        }
        if(seq_num[current_picture]!=0)
            if((seq_num[current_picture]-prev_output_seq)!=1)
                fprintf(err,"ERROR: picture order error! previous=%i, current=%i\n",
                        prev_output_seq, seq_num[current_picture]);

        #ifdef TextMode
            printf("--------------PICTURE %i-------------\n", picture_count);
            printf("temporal_ref=%i\n", temporal_ref);
            printf("picture_coding_type=%i\n", picture_coding_type);
            printf("vbv_delay=%i\n", vbv_delay);
            printf("current_picture=%i\n", current_picture);
            printf("future_picture=%i\n", future_picture);
            printf("past_picture=%i\n", past_picture);
        #endif

        do{
            slice();
        }while(input->next32()>=0x00000101 &&
               input->next32()<=0x000001AF);
        break;

    case sequence_end_code :
        return 0;

    default:
        fprintf(err,"ERROR: Unexpected start code!\n");
        input->getn(32);
    }
    return 1;
}

void MpegClass::slice(void){
    slice_v_pos= input->getn(32)&0xFF;
    if(slice_v_pos==0 || slice_v_pos>0xAF)
        fprintf(err,"ERROR: slice start position error %i\n",
                    slice_v_pos);
    q_scale=(int)input->getn(5);
    #ifdef TextMode
        printf("---- start of slice ----\n");
        printf("slice_v_pos=%i q_scale=%i\n",slice_v_pos,q_scale);
    #endif
    while(input->getn(1)==1)
        input->getn(8);

    //Variables initialization
    prev_mb_addr=(slice_v_pos-1)*mb_width-1;
    dct_dc_y_past=dct_dc_cb_past=dct_dc_cr_past=1024;
    past_intra_addr=-2;
    recon_down_for_prev =recon_right_for_prev =0;
    recon_down_back_prev=recon_right_back_prev=0;

    do{
        macroblock();
    }while(input->next23() != 0x000000);
    input->next_start_code();
}


void MpegClass::macroblock(void){

    //stuffing and escape are processed in the following function
    mb_addr_inc=input->get_mb_address_increment();
    mb_addr=prev_mb_addr+mb_addr_inc;
    mb_type=input->get_mb_type(picture_coding_type);

    if(mb_addr_inc==0 || mb_type==0){
        fprintf(err,"ERROR: mb_addr_inc==0 or mb_type==0\n");
        input->next_start_code();
        return;
    }

    #ifdef TextMode 
        printf("--macroblock %i--\n", mb_addr);
        printf("mb_type=%X\n", mb_type);
    #endif

    switch(picture_coding_type){

    case 1: // for I-picture
            // no skipped macroblocks
        if(mb_addr_inc>1 && ((prev_mb_addr+1)%mb_width) ){
            fprintf(err,"ERROR: I-picture shouldn't have skipped macroblocks\n");
            //input->next_start_code();
            //goto EndOfSlice;
        }
        if(mb_type & macroblock_quant){
            q_scale=(int)input->getn(5);
            #ifdef TextMode
                printf("q_scale=%i\n", q_scale);
            #endif
        }
        if(mb_type & (macroblock_motion_forward | 
                      macroblock_motion_backward  ))
            fprintf(err,"ERROR: There shouldn't be motion vector in I-picture\n");
        if(mb_type & macroblock_pattern)
            fprintf(err,"ERROR: I-picture shouldn't have pattern code!\n");
        if(!(mb_type & macroblock_intra))
            fprintf(err,"ERROR: I-picture should be all intra macroblocks!\n");

        intra_macroblock();

        break;

    case 2: // for P-picture
        //deal with skiped macroblock
        if( (mb_addr_inc>1) && ((prev_mb_addr+1)%mb_width) ){
            for(prev_mb_addr++;prev_mb_addr<mb_addr;prev_mb_addr++)
                deal_p_skipped_macroblock(prev_mb_addr);
            recon_right_for_prev=recon_down_for_prev=0;
        }
        if(mb_type & macroblock_quant)
            q_scale=(int)input->getn(5);
        if(mb_type & macroblock_motion_forward){
            motion_horizontal_forward_code=input->get_mv();
            if((forward_f!=1)&&(motion_horizontal_forward_code!=0))
                motion_horizontal_forward_r=input->getn(forward_r_size);
            motion_vertical_forward_code=input->get_mv();
            if((forward_f!=1)&&(motion_vertical_forward_code!=0))
                motion_vertical_forward_r=input->getn(forward_r_size);
        }else{
            recon_right_for_prev=recon_down_for_prev=0;
        }
        if(mb_type & macroblock_motion_backward)
            fprintf(err,"ERROR: P-picture shouldn't use backward motion vector\n");
        if(mb_type & macroblock_pattern)
            cbpattern=input->get_cbp();
        else
            cbpattern=0;
        if(mb_type & macroblock_intra)
            intra_macroblock();
        else
            p_macroblock_in_p_picture();
        break;

    case 3: // for B-picture
        //deal with skiped macroblock
        if( (mb_addr_inc>1) && ((prev_mb_addr+1)%mb_width) ){
            for(prev_mb_addr++;prev_mb_addr<mb_addr;prev_mb_addr++)
                deal_b_skipped_macroblock(prev_mb_addr);
        }
        if(mb_type & macroblock_quant)
            q_scale=(int)input->getn(5);
        if(mb_type & macroblock_motion_forward){
            motion_horizontal_forward_code=input->get_mv();
            if((forward_f!=1)&&(motion_horizontal_forward_code!=0))
                motion_horizontal_forward_r=input->getn(forward_r_size);
            motion_vertical_forward_code=input->get_mv();
            if((forward_f!=1)&&(motion_vertical_forward_code!=0))
                motion_vertical_forward_r=input->getn(forward_r_size);
        }
        if(mb_type & macroblock_motion_backward){
            motion_horizontal_backward_code=input->get_mv();
            if((backward_f!=1)&&(motion_horizontal_backward_code!=0))
                motion_horizontal_backward_r=input->getn(backward_r_size);
            motion_vertical_backward_code=input->get_mv();
            if((backward_f!=1)&&(motion_vertical_backward_code!=0))
                motion_vertical_backward_r=input->getn(backward_r_size);
        }
        if(mb_type & macroblock_pattern)
            cbpattern=input->get_cbp();
        else
            cbpattern=0;
        if(mb_type & macroblock_intra){
            intra_macroblock();
            recon_down_for_prev =recon_right_for_prev =0;
            recon_down_back_prev=recon_right_back_prev=0;
        }else
            p_macroblock_in_b_picture();
        break;

    case 4: // for D-picture

        break;

    default:
        fprintf(err,"ERROR: do not have this type of picture %i\n",
                     picture_coding_type);
    }
    prev_mb_addr=mb_addr;
}

void MpegClass::intra_macroblock(void)
{
    int run,level,i,j, dc_diff,dc_size;
	for(i=0 ; i<4; i++){

        memset(IDCT_Source, 0, sizeof(int[64]));

        dc_size=input->get_dc_size_l();
        if(dc_size<0){
            fprintf(err,"ERROR: dc_size<0\n");
        }
        dc_diff=input->getn(dc_size);
        if(dc_diff & (1<<(dc_size-1))) IDCT_Source[0]=dc_diff;
        else IDCT_Source[0]=(-1<<(dc_size))|(dc_diff+1);
        
        IDCT_Source[0] *= 8;
        if(i==0){
            if(mb_addr-past_intra_addr>1)
                IDCT_Source[0] += 1024;
            else
                IDCT_Source[0] += dct_dc_y_past;
            dct_dc_y_past=IDCT_Source[0];
        }else
            dct_dc_y_past=IDCT_Source[0]+=dct_dc_y_past;

        IDCT_Source[0] *= i_qTab[0];

        for(j=1 ; /*j<64*/ ; j++){
		    input->get_dct_next(run,level);
	        j += run ;
            if(j<0){
                fprintf(err, "ERROR: j<0 j=%i\n",j);
            }
            if(j>64){
                fprintf(err, "ERROR: j>64 j=%i\n",j);
            }
	        if( level == 0 && run==0 )
	           break ;
			else {
               level=(2*level*q_scale*i_qTab[j])/16;
               if((level&1)==0)
                   level-=Sign(level);
               //if(level>2047)  level= 2047;
               //if(level<-2048) level=-2048;
		       if(j>63) j=63;
               IDCT_Source[IDCT_P[j]] = level;
			}
		}
		ScaleIDCT(MACROBLOCK[i]) ;
	}
    for(i=4; i<6; i++){

        memset(IDCT_Source, 0, sizeof(int[64]));

        dc_size=input->get_dc_size_c();
        if(dc_size<0){
            fprintf(err, "ERROR: dc_size<0 dc_size=%i\n", dc_size);
        }
        dc_diff=input->getn(dc_size);
        if(dc_diff & (1<<(dc_size-1))) IDCT_Source[0]=dc_diff;
        else IDCT_Source[0]=(-1<<(dc_size))|(dc_diff+1);
        
        IDCT_Source[0] *= 8;
        if(mb_addr-past_intra_addr>1)
            IDCT_Source[0] += 1024;
        else{
            if(i==4) IDCT_Source[0] += dct_dc_cb_past;
            else IDCT_Source[0] += dct_dc_cr_past;
        }
        if(i==4) dct_dc_cb_past=IDCT_Source[0];
        else dct_dc_cr_past=IDCT_Source[0];

        IDCT_Source[0] *= i_qTab[0];

        for(j=1 ; j<64 ; j++){
		    input->get_dct_next(run,level);
	        j += run ;
	        if( level == 0 && run==0 )
	           break ;
			else {
               level=(2*level*q_scale*i_qTab[j])/16;
               if((level&1)==0)
                   level-=Sign(level);
               //if(level>2047)  level= 2047;
               //if(level<-2048) level=-2048;
		       if(j>63) j=63;
               IDCT_Source[IDCT_P[j]] = level;
			}
		}
		ScaleIDCT(MACROBLOCK[i]) ;
	}
    past_intra_addr=mb_addr;

    prev_option=0; //previous option for output(....)
    output(0, mb_addr);
}


//predictive macroblock in P-picture
void MpegClass::p_macroblock_in_p_picture(void)
{
    int run,level,i,j;
	for(i=0 ; i<6; i++){

        if( (cbpattern & (1<<(5-i))) == 0 ){
            memset(MACROBLOCK[i], 0, sizeof(int[64]));
            continue;
        }

        memset(IDCT_Source, 0, sizeof(int[64]));

        input->get_dct_first(run,level);
        j=run;
        if(level!=0){
            level=((2*level+Sign(level))*q_scale*n_qTab[j])/16;
            if((level&1)==0)
                level-=Sign(level);
            //if(level>2047)  level= 2047;
            //if(level<-2048) level=-2048;
        }
        if(j>63) j=63;
		IDCT_Source[IDCT_P[j]] = level;

        for(j++ ; j<64 ; j++){
		    input->get_dct_next(run,level);
	        j += run ;
	        if( level == 0 && run==0 )
	           break ;
			else {
              if(level!=0){
                level=((2*level+Sign(level))*q_scale*n_qTab[j])/16;
                if((level&1)==0)
                    level-=Sign(level);
                //if(level>2047)  level= 2047;
                //if(level<-2048) level=-2048;
              }
              if(j>63) j=63;
		      IDCT_Source[IDCT_P[j]] = level;
			}
		}
		ScaleIDCT(MACROBLOCK[i]) ;
	}

    if((mb_type & macroblock_motion_forward)==0){
        output(4,mb_addr);
        return;
    }

    int c_h_f_r, right_big, right_little, down_big, down_little,
        c_v_f_r, max, min;

    if(forward_f==1 || motion_horizontal_forward_code==0)
        c_h_f_r=0;
    else
        c_h_f_r=forward_f-1-motion_horizontal_forward_r;
    if(forward_f==1 || motion_vertical_forward_code==0)
        c_v_f_r=0;
    else
        c_v_f_r=forward_f-1-motion_vertical_forward_r;
    right_little=motion_horizontal_forward_code*forward_f;
    if(right_little==0)
        right_big=0;
    else{
        if(right_little>0){
            right_little-=c_h_f_r;
            right_big=right_little-32*forward_f;
        }else{
            right_little+=c_h_f_r;
            right_big=right_little+32*forward_f;
        }
    }
    down_little=motion_vertical_forward_code*forward_f;
    if(down_little==0)
        down_big=0;
    else{
        if(down_little>0){
            down_little-=c_v_f_r;
            down_big=down_little-32*forward_f;
        }else{
            down_little+=c_v_f_r;
            down_big=down_little+32*forward_f;
        }
    }

    int new_vector;

    max=(16*forward_f)-1;
    min=(-16*forward_f);

    new_vector=recon_right_for_prev+right_little;
    if(new_vector<=max && new_vector>=min)
        recon_right_for=recon_right_for_prev+right_little;
    else
        recon_right_for=recon_right_for_prev+right_big;
    recon_right_for_prev=recon_right_for;
    if( full_pel_forward_vector ) recon_right_for <<= 1;

    new_vector=recon_down_for_prev+down_little;
    if(new_vector<=max && new_vector>=min)
        recon_down_for=recon_down_for_prev+down_little;
    else
        recon_down_for=recon_down_for_prev+down_big;
    recon_down_for_prev=recon_down_for;
    if( full_pel_forward_vector ) recon_down_for <<= 1;

    output(1, mb_addr);
}

//predictive macroblock in B-picture
void MpegClass::p_macroblock_in_b_picture(void){
    int run,level,i,j;
	for(i=0 ; i<6; i++){

        if( (cbpattern & (1<<(5-i))) == 0 ){
            memset(MACROBLOCK[i], 0, sizeof(int[64]));
            continue;
        }

        memset(IDCT_Source, 0, sizeof(int[64]));

        input->get_dct_first(run,level);
        j=run;
        if(level!=0){
            level=((2*level+Sign(level))*q_scale*n_qTab[j])/16;
            if((level&1)==0)
                level-=Sign(level);
            //if(level>2047)  level= 2047;
            //if(level<-2048) level=-2048;
        }
        if(j>63) j=63;
		IDCT_Source[IDCT_P[j]] = level;

        for(j++ ; j<64 ; j++){
		    input->get_dct_next(run,level);
	        j += run ;
	        if( level == 0 && run==0 )
	           break ;
			else {
              if(level!=0){
                level=((2*level+Sign(level))*q_scale*n_qTab[j])/16;
                if((level&1)==0)
                    level-=Sign(level);
                //if(level>2047)  level= 2047;
                //if(level<-2048) level=-2048;
              }
              if(j>63) j=63;
		      IDCT_Source[IDCT_P[j]] = level;
			}
		}
		ScaleIDCT(MACROBLOCK[i]) ;
	}

    if((mb_type & macroblock_motion_forward)==0 ){
        recon_right_for=recon_right_for_prev;
        recon_down_for=recon_down_for_prev;
    }else{
        int c_h_f_r, right_big, right_little, down_big, down_little,
            c_v_f_r, max, min;

        if(forward_f==1 || motion_horizontal_forward_code==0)
            c_h_f_r=0;
        else
            c_h_f_r=forward_f-1-motion_horizontal_forward_r;
        if(forward_f==1 || motion_vertical_forward_code==0)
            c_v_f_r=0;
        else
            c_v_f_r=forward_f-1-motion_vertical_forward_r;
        right_little=motion_horizontal_forward_code*forward_f;
        if(right_little==0)
            right_big=0;
        else{
            if(right_little>0){
                right_little-=c_h_f_r;
                right_big=right_little-32*forward_f;
            }else{
                right_little+=c_h_f_r;
                right_big=right_little+32*forward_f;
            }
        }
        down_little=motion_vertical_forward_code*forward_f;
        if(down_little==0)
            down_big=0;
        else{
            if(down_little>0){
                down_little-=c_v_f_r;
                down_big=down_little-32*forward_f;
            }else{
                down_little+=c_v_f_r;
                down_big=down_little+32*forward_f;
            }
        }

        int new_vector;

        max=(16*forward_f)-1;
        min=(-16*forward_f);

        new_vector=recon_right_for_prev+right_little;
        if(new_vector<=max && new_vector>=min)
            recon_right_for=recon_right_for_prev+right_little;
        else
            recon_right_for=recon_right_for_prev+right_big;
        recon_right_for_prev=recon_right_for;
        if( full_pel_forward_vector ) recon_right_for <<= 1;

        new_vector=recon_down_for_prev+down_little;
        if(new_vector<=max && new_vector>=min)
            recon_down_for=recon_down_for_prev+down_little;
        else
            recon_down_for=recon_down_for_prev+down_big;
        recon_down_for_prev=recon_down_for;
        if( full_pel_forward_vector ) recon_down_for <<= 1;
    }

    if((mb_type & macroblock_motion_backward)==0){
        recon_right_back=recon_right_back_prev;
        recon_down_back=recon_down_back_prev;
    }else{
        int c_h_f_r, right_big, right_little, down_big, down_little,
            c_v_f_r, max, min;

        if(backward_f==1 || motion_horizontal_backward_code==0)
            c_h_f_r=0;
        else
            c_h_f_r=backward_f-1-motion_horizontal_backward_r;
        if(backward_f==1 || motion_vertical_backward_code==0)
            c_v_f_r=0;
        else
            c_v_f_r=backward_f-1-motion_vertical_backward_r;
        right_little=motion_horizontal_backward_code*backward_f;
        if(right_little==0)
            right_big=0;
        else{
            if(right_little>0){
                right_little-=c_h_f_r;
                right_big=right_little-32*backward_f;
            }else{
                right_little+=c_h_f_r;
                right_big=right_little+32*backward_f;
            }
        }
        down_little=motion_vertical_backward_code*backward_f;
        if(down_little==0)
            down_big=0;
        else{
            if(down_little>0){
                down_little-=c_v_f_r;
                down_big=down_little-32*backward_f;
            }else{
                down_little+=c_v_f_r;
                down_big=down_little+32*backward_f;
            }
        }

        int new_vector;

        max=(16*backward_f)-1;
        min=(-16*backward_f);

        new_vector=recon_right_back_prev+right_little;
        if(new_vector<=max && new_vector>=min)
            recon_right_back=recon_right_back_prev+right_little;
        else
            recon_right_back=recon_right_back_prev+right_big;
        recon_right_back_prev=recon_right_back;
        if( full_pel_backward_vector ) recon_right_back <<= 1;

        new_vector=recon_down_back_prev+down_little;
        if(new_vector<=max && new_vector>=min)
            recon_down_back=recon_down_back_prev+down_little;
        else
            recon_down_back=recon_down_back_prev+down_big;
        recon_down_back_prev=recon_down_back;
        if( full_pel_backward_vector ) recon_down_back <<= 1;    
    }
    if(mb_type & macroblock_motion_forward){
        if(mb_type & macroblock_motion_backward){
            prev_option=3;
            output(3,mb_addr);
        }else{
            prev_option=1;
            output(1,mb_addr);
        }
    }else{
        if(mb_type & macroblock_motion_backward){
            prev_option=2;
            output(2,mb_addr);
        }else{
            if(!(mb_type & macroblock_intra))
                fprintf(err,"ERROR: mb_type=0x%X\n", mb_type);
            prev_option=0;
            output(0,mb_addr);
        }
    }
}

void MpegClass::deal_p_skipped_macroblock(int mb_addr_to_fill)
{
    output(12, mb_addr_to_fill); //motion vector 0,0 and no error
}

void MpegClass::deal_b_skipped_macroblock(int mb_addr_to_fill)
{
    if(prev_option==0)
        fprintf(err,"ERROR: The macroblock after intra coded one shouldn't skip!\n");
    output(prev_option+8,mb_addr_to_fill);
}


/*MpegClass::compute_motion:
    rr : recon_right_...
    rd : recon_down_...
    x,y: posion at the start of the macroblock
    pel_new : the currently being decoded picture
    pel_ref : the referenced picture
*/
inline void 
MpegClass::compute_motion(int rr, int rd, int x, int y,
                          int* pel_new, int* pel_ref){
    int right, down, right_half, down_half;
    int i,j,ptr, old_x, image_size;
    image_size=h_size*v_size;
    old_x=x;
    right=rr>>1;
    down=rd>>1;
    right_half=rr-2*right;
    down_half=rd-2*down;
    if((!right_half) && (!down_half)){
        for(i=0; i<16; i++, y++){
            ptr=y*h_size;
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy=image_size-1;
                    pel_new[ptr+x]=pel_ref[tempxy];
                }
            }
        }
    }else if((!right_half) && down_half){
        for(i=0; i<16; i++, y++){
            ptr=y*h_size;
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy+=h_size;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    pel_new[ptr+x]=YCbCr((GET_Y(ref1)+GET_Y(ref2))/2,
                                         (GET_Cb(ref1)+GET_Cb(ref2))/2,
                                         (GET_Cr(ref1)+GET_Cr(ref2))/2);
                }
            }
        }
    }else if(right_half && (!down_half)){
        for(i=0; i<16; i++, y++){
            ptr=y*h_size;
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy > image_size) tempxy=image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy > image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    pel_new[ptr+x]=YCbCr((GET_Y(ref1)+GET_Y(ref2))/2,
                                         (GET_Cb(ref1)+GET_Cb(ref2))/2,
                                         (GET_Cr(ref1)+GET_Cr(ref2))/2);
                }
            }
        }
    }else{
        for(i=0; i<16; i++, y++){
            ptr=y*h_size;
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy = image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    tempxy+=h_size-1;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref3=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref4=pel_ref[tempxy];
                    pel_new[ptr+x]=YCbCr((GET_Y(ref1)+GET_Y(ref2)+GET_Y(ref3)+GET_Y(ref4))/4,
                                         (GET_Cb(ref1)+GET_Cb(ref2)+GET_Cb(ref3)+GET_Cb(ref4))/4,
                                         (GET_Cr(ref1)+GET_Cr(ref2)+GET_Cr(ref3)+GET_Cr(ref4))/4);
                }
            }
        }
    }
}

/*MpegClass::compute_motion_temp:
    rr : recon_right_...
    rd : recon_down_...
    x,y: posion at the start of the macroblock
    pel_new : int[16*16]; 
    pel_ref : the referenced picture
*/
inline void 
MpegClass::compute_motion_temp(int rr, int rd, int x, int y,
                               int* pel_new, int* pel_ref){
    int right, down, right_half, down_half;
    int i,j, old_x, image_size;
    image_size=h_size*v_size;
    old_x=x;
    right=rr>>1;
    down=rd>>1;
    right_half=rr-2*right;
    down_half=rd-2*down;
    if((!right_half) && (!down_half)){
        for(i=0; i<16; i++, y++){
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy=image_size-1;
                    pel_new[16*i+j]=pel_ref[tempxy];
                }
            }
        }
    }else if((!right_half) && down_half){
        for(i=0; i<16; i++, y++){
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy+=h_size;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    pel_new[16*i+j]=YCbCr((GET_Y(ref1)+GET_Y(ref2))/2,
                                         (GET_Cb(ref1)+GET_Cb(ref2))/2,
                                         (GET_Cr(ref1)+GET_Cr(ref2))/2);
                }
            }
        }
    }else if(right_half && (!down_half)){
        for(i=0; i<16; i++, y++){
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy > image_size) tempxy=image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy > image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    pel_new[16*i+j]=YCbCr((GET_Y(ref1)+GET_Y(ref2))/2,
                                         (GET_Cb(ref1)+GET_Cb(ref2))/2,
                                         (GET_Cr(ref1)+GET_Cr(ref2))/2);
                }
            }
        }
    }else{
        for(i=0; i<16; i++, y++){
            x=old_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    int tempxy=(y+down)*h_size+x+right;
                    if(tempxy>image_size) tempxy = image_size-1;
                    int ref1=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref2=pel_ref[tempxy];
                    tempxy+=h_size-1;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref3=pel_ref[tempxy];
                    tempxy++;
                    if(tempxy>image_size) tempxy=image_size-1;
                    int ref4=pel_ref[tempxy];
                    pel_new[16*i+j]=YCbCr((GET_Y(ref1)+GET_Y(ref2)+GET_Y(ref3)+GET_Y(ref4))/4,
                                         (GET_Cb(ref1)+GET_Cb(ref2)+GET_Cb(ref3)+GET_Cb(ref4))/4,
                                         (GET_Cr(ref1)+GET_Cr(ref2)+GET_Cr(ref3)+GET_Cr(ref4))/4);
                }
            }
        }
    }
}

void MpegClass::output(int option, int mb_addr_to_fill){
    int start_x,start_y,x,y,i,j,k;
    int *YImage, *CbImage, *CrImage;
    start_x = (mb_addr_to_fill%mb_width) * 16;
    start_y = (mb_addr_to_fill/mb_width) * 16;

    if(option==0){  //for intra-coded macroblock
        CbImage=MACROBLOCK[4];
        CrImage=MACROBLOCK[5];
        for(i=0; i<4; i++){
            YImage=MACROBLOCK[i];
            if(i<2){ y=start_y; k=0;}
            else{ y=start_y+8; k=32;}
            if(i%2){ x=start_x+8; k+=4;}
            else x=start_x;
            for(j=0; j<64;){
                if(x<h_size && y<v_size)
                    pel[y*h_size+x]=
                        YCbCr(YImage[j],CbImage[k],CrImage[k]);
                if(x<h_size && y<v_size)
                    pel[(y+1)*h_size+x]=
                        YCbCr(YImage[j+8],CbImage[k],CrImage[k]);
                x++; j++;
                if(x<h_size && y<v_size)
                    pel[y*h_size+x]=
                        YCbCr(YImage[j],CbImage[k],CrImage[k]);
                if(x<h_size && y<v_size)
                    pel[(y+1)*h_size+x]=
                        YCbCr(YImage[j+8],CbImage[k],CrImage[k]);
                x++; j++;
                k++;
                if((x%8)==0){
                     x-=8; y+=2; j+=8; k+=4;
                }
            }
        }
        return;
    }
        
    switch(option%8){
    case 1: // for forward motion vector
        compute_motion(recon_right_for, recon_down_for,
                       start_x, start_y, pel, pel_past);
        break;
    case 2: // for backward motion vector
        compute_motion(recon_right_back, recon_down_back,
                       start_x, start_y, pel, pel_future);
        break;
    case 3:
        int pel_for[256];
        int pel_back[256];
        compute_motion_temp(recon_right_for, recon_down_for,
                            start_x, start_y, pel_for , pel_past);
        compute_motion_temp(recon_right_back, recon_down_back,
                            start_x, start_y, pel_back, pel_future);
        y=start_y;
        int ref1,ref2;
        for(i=0; i<16; i++, y++){
            x=start_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size){
                    ref1=pel_for[16*i+j];
                    ref2=pel_back[16*i+j];
                    pel[y*h_size+x]=YCbCr((GET_Y(ref1)+GET_Y(ref2))/2,
                                         (GET_Cb(ref1)+GET_Cb(ref2))/2,
                                         (GET_Cr(ref1)+GET_Cr(ref2))/2);
                }
            }
        }
        break;
    case 4:
        y=start_y;
        for(i=0; i<16; i++, y++){
            x=start_x;
            for(j=0; j<16; j++, x++){
                if(x<h_size && y<v_size)
                    pel[y*h_size+x]=pel_past[y*h_size+x];
            }
        }
        break;
    default:
        fprintf(err,"ERROR: output(...) option=%i\n", option);
    }
    if(option>8) return; // if no errors are coded.

    CbImage=MACROBLOCK[4];
    CrImage=MACROBLOCK[5];
    for(i=0; i<4; i++){
        YImage=MACROBLOCK[i];
        if(i<2){ y=start_y; k=0;}
        else{ y=start_y+8; k=32;}
        if(i%2){ x=start_x+8; k+=4;}
        else x=start_x;
        for(j=0; j<64;){
            int pel_temp;
            if(x<h_size && y<v_size){
                pel_temp=pel[y*h_size+x];
                pel[y*h_size+x]=
                    YCbCr(GET_Y( pel_temp)+YImage[j],
                          GET_Cb(pel_temp)+CbImage[k],
                          GET_Cr(pel_temp)+CrImage[k]);
            }
            if(x<h_size && y<v_size){
                pel_temp=pel[(y+1)*h_size+x];
                pel[(y+1)*h_size+x]=
                    YCbCr(GET_Y( pel_temp)+YImage[j+8],
                          GET_Cb(pel_temp)+CbImage[k],
                          GET_Cr(pel_temp)+CrImage[k]);
            }
            x++; j++;
            if(x<h_size && y<v_size){
                pel_temp=pel[y*h_size+x];
                pel[y*h_size+x]=
                    YCbCr(GET_Y( pel_temp)+YImage[j],
                          GET_Cb(pel_temp)+CbImage[k],
                          GET_Cr(pel_temp)+CrImage[k]);
            }
            if(x<h_size && y<v_size){
                pel_temp=pel[(y+1)*h_size+x];
                pel[(y+1)*h_size+x]=
                    YCbCr(GET_Y( pel_temp)+YImage[j+8],
                          GET_Cb(pel_temp)+CbImage[k],
                          GET_Cr(pel_temp)+CrImage[k]);
            }
            x++; j++;
            k++;
            if((x%8)==0){
                 x-=8; y+=2; j+=8; k+=4;
            }
        }
    }
}


void YCbCr_to_RGB(int y,int u,int v,char* BGR_IMAGE){
    int tempB,tempG,tempR;
    u-=128; v-=128;
    tempR=y+((91881*v)>>16);
    if(tempR<0) *(BGR_IMAGE++)=0;
    else if(tempR>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempR;

    tempG=y-((22554*u+46802*v)>>16);
    if(tempG<0) *(BGR_IMAGE++)=0; 
    else if(tempG>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempG;
    
    tempB=y+((116130*u)>>16);
    if(tempB<0) *(BGR_IMAGE)=0; 
    else if(tempB>255) *(BGR_IMAGE)='\xFF';
    else *(BGR_IMAGE)=tempB;
}

void Image_YCbCr_to_RGB(int* YCbCrImage, char* BGRImage, 
                        int h_size, int v_size){
    int YCbCrSize=h_size*v_size;
    int i,j,temp;
    for(i=0, j=0; i<YCbCrSize; i++, j+=3){
        temp=YCbCrImage[i];
        YCbCr_to_RGB(GET_Y(temp),GET_Cb(temp),GET_Cr(temp),BGRImage+j);
    }
}

void YCbCr_to_BGR(int y,int u,int v,char* BGR_IMAGE){
    int tempB,tempG,tempR;
    u-=128; v-=128;
    tempB=y+((116130*u)>>16);
    if(tempB<0) *(BGR_IMAGE++)=0; 
    else if(tempB>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempB;

    tempG=y-((22554*u+46802*v)>>16);
    if(tempG<0) *(BGR_IMAGE++)=0; 
    else if(tempG>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempG;
    
    tempR=y+((91881*v)>>16);
    if(tempR<0) *(BGR_IMAGE)=0;
    else if(tempR>255) *(BGR_IMAGE)='\xFF';
    else *(BGR_IMAGE)=tempR;
}

void Image_YCbCr_to_BGR(int* YCbCrImage, char* BGRImage, 
                        int h_size, int v_size){
    int YCbCrSize=h_size*v_size;
    int i,j,temp;
    for(i=0, j=0; i<YCbCrSize; i++, j+=3){
        temp=YCbCrImage[i];
        YCbCr_to_BGR(GET_Y(temp),GET_Cb(temp),GET_Cr(temp),BGRImage+j);
    }
}


/*
inline void MpegClass::get_l_motion_vector_for(void){
    right_for=recon_right_for>>1;
    down_for=recon_down_for>>1;
    right_half_for=recon_right_for-2*right_for;
    down_half_for=recon_down_for-2*down_for;
}

inline void MpegClass::get_c_motion_vector_for(void){
    right_for=(recon_right_for/2)>>1;
    down_for=(recon_down_for/2)>>1;
    right_half_for=(recon_right_for/2)-2*right_for;
    down_half_for=(recon_down_for/2)-2*down_for;
}
inline void MpegClass::get_l_motion_vector_back(void){
    right_back=recon_right_back>>1;
    down_back=recon_down_back>>1;
    right_half_back=recon_right_back-2*right_back;
    down_half_back=recon_down_back-2*down_back;
}

inline void MpegClass::get_c_motion_vector_back(void){
    right_back=(recon_right_back/2)>>1;
    down_back=(recon_down_back/2)>>1;
    right_half_back=(recon_right_back/2)-2*right_back;
    down_half_back=(recon_down_back/2)-2*down_back;
}
*/
/*
void MpegClass::test_macroblock(void){
    mb_addr_inc=input->get_mb_address_increment();
    mb_addr=prev_mb_addr+mb_addr_inc;
    mb_type=input->get_mb_type(picture_coding_type);

    printf("--macroblock %i--\n", mb_addr);
    printf("mb_type=%X\n", mb_type);

    if(mb_type & macroblock_quant){
        q_scale=(int)input->getn(5);
        printf("q_scale=%i\n", q_scale);
    }
    if(mb_type & macroblock_motion_forward){
        motion_horizontal_forward_code=input->get_mv();
        if((forward_f!=1)&&(motion_horizontal_forward_code!=0))
            motion_horizontal_forward_r=input->getn(forward_r_size);
        motion_vertical_forward_code=input->get_mv();
        if((forward_f!=1)&&(motion_vertical_forward_code!=0))
            motion_vertical_forward_r=input->getn(forward_r_size);
    }
    if(mb_type & macroblock_motion_backward){
        motion_horizontal_backward_code=input->get_mv();
        if((backward_f!=1)&&(motion_horizontal_backward_code!=0))
            motion_horizontal_backward_r=input->getn(backward_r_size);
        motion_vertical_backward_code=input->get_mv();
        if((backward_f!=1)&&(motion_vertical_backward_code!=0))
            motion_vertical_backward_r=input->getn(backward_r_size);
    }
    if(mb_type & macroblock_pattern)
        cbpattern=input->get_cbp();
    else
        cbpattern=0;
    if(mb_type & macroblock_intra) cbpattern=0x3F;
    printf("cbpattern=0x%X\n", cbpattern);
    for(int i=0; i<6; i++)
        test_block(i);
    if(picture_coding_type==4)
        input->getn(1);
    prev_mb_addr=mb_addr;
}

void MpegClass::test_block(int i){
    int temp, run, level;
    if(cbpattern & (1<<(5-i))){
        if(macroblock_intra){
            if(i<4){
                temp=input->get_dc_size_l();
                //printf("dct_dc_size_l=%i\n", temp);
                if(temp != 0){
                    temp=input->getn(temp);
                    //printf("dct_dc_diff=%i\n", temp);
                }
            }else{
                temp=input->get_dc_size_c();
                //printf("dct_dc_size_c=%i\n", temp);
                if(temp != 0){
                    temp=input->getn(temp);
                    //printf("dct_dc_diff=%i\n", temp);
                }
            }
        }else{
            input->get_dct_first(run, level);
            //printf("dct_first: run=%i, level=%i\n", run, level);
        }
        if(picture_coding_type !=4){
            while(input->nextn(2)!=0x2){
                input->get_dct_next(run, level);
                //printf("dct_next: run%i, level=%i\n", run, level);
            }
            input->getn(2);
            //printf("end of block\n");
        }
    }
}

*/
/*
void YCbCr_to_RGB(int y,int u,int v,char* BGR_IMAGE){
    int NEW_Y=y+128;
    int tempB,tempG,tempR;
    if(y<-128) NEW_Y=0;

    tempR=NEW_Y+((91881*v)>>16);
    if(tempR<0) *(BGR_IMAGE++)=0;
    else if(tempR>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempR;

    tempG=NEW_Y-((22554*u+46802*v)>>16);
    if(tempG<0) *(BGR_IMAGE++)=0; 
    else if(tempG>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempG;
    
    tempB=NEW_Y+((116130*u)>>16);
    if(tempB<0) *(BGR_IMAGE)=0; 
    else if(tempB>255) *(BGR_IMAGE)='\xFF';
    else *(BGR_IMAGE)=tempB;
}

void YCbCr_to_BGR(int y,int u,int v,char* BGR_IMAGE){
    int NEW_Y=y+128;
    int tempB,tempG,tempR;
    if(y<-128) NEW_Y=0;
    tempB=NEW_Y+((116130*u)>>16);
    if(tempB<0) *(BGR_IMAGE++)=0; 
    else if(tempB>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempB;

    tempG=NEW_Y-((22554*u+46802*v)>>16);
    if(tempG<0) *(BGR_IMAGE++)=0; 
    else if(tempG>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempG;
    
    tempR=NEW_Y+((91881*v)>>16);
    if(tempR<0) *(BGR_IMAGE)=0;
    else if(tempR>255) *(BGR_IMAGE)='\xFF';
    else *(BGR_IMAGE)=tempR;
}
*/