#include "bi_io.h"

#define picture_start_code      0x00000100
#define user_data_start_code    0x000001B2
#define sequence_header_code    0x000001B3
#define sequence_error_code     0x000001B4
#define extension_start_code    0x000001B5
#define sequence_end_code       0x000001B7
#define group_start_code        0x000001B8

#define Y_SHIFT     20
#define Cb_SHIFT    10
#define Y_MASK      0xFFF00000
#define Cb_MASK     0x000FFC00
#define Cr_MASK     0x000003FF
#define YCbCr(y,cb,cr) ((y<<Y_SHIFT) | \
                        ((cb << Cb_SHIFT) & Cb_MASK) |\
                        (cr & Cr_MASK))
#define GET_Y(temp)  (temp>>Y_SHIFT)    // signed shift
#define GET_Cb(temp) ((temp<<12) >> 22) // signed shift
#define GET_Cr(temp) ((temp<<22) >> 22) // signed shift


class MpegClass{
  //for sequence_header layer
  public:
    int h_size;
    int v_size;
  private:
    int pel_aspect_ratio;
    int picture_rate;
    int bit_rate;
    int vbv_buffer_size;
    int constrained_parameter_flag;
    int i_qTab[64]; //in zz order
    int n_qTab[64]; //in zz order

  //for group_of_pictures layer
    unsigned int time_code;
    int closed_gop;
    int broken_link;

  //for picture layer
    int temporal_ref;
    int picture_coding_type;
    int vbv_delay;
    int full_pel_forward_vector;
    int forward_f_code;
    int full_pel_backward_vector;
    int backward_f_code;
    int forward_f;
    int forward_r_size;
    int backward_f;
    int backward_r_size;

  //for slice layer
    int slice_v_pos;
    int q_scale;

  //for macroblock layer
    int mb_addr_inc;
    int mb_addr;
    int mb_type;
    int motion_horizontal_forward_code;
    int motion_horizontal_forward_r;
    int motion_vertical_forward_code;
    int motion_vertical_forward_r;
    int motion_horizontal_backward_code;
    int motion_horizontal_backward_r;
    int motion_vertical_backward_code;
    int motion_vertical_backward_r;
    int cbpattern;

  //Variables to make picture reference fast
    int* pel;        // currently being decoded picture.
    int* pel_past;   // past reference
    int* pel_future; // future reference

    int picture_count; //picture count in a group of pictures
    int prev_output_seq;

  //Variables for position
    int mb_width, mb_height;//Picture size in Macroblock unit 
    int mb_row, mb_column;  //Current Position to decode, 
                            // in Macroblock unit.

  //Variable to store a Macroblock
    int MACROBLOCK[6][64];

  //Input Object
    bi_io_class *input;

  //Motion Vectors
    int recon_right_for,  recon_down_for ;
    int recon_right_back, recon_down_back;

  //Variables to store previous information
    int prev_mb_addr; //previous macroblock address
    int past_intra_addr;  //previous intra-coded macroblock address
    int dct_dc_y_past;   //for intra-coded macroblock
    int dct_dc_cb_past;  //for intra-coded macroblock
    int dct_dc_cr_past;  //for intra-coded macroblock
    //for predictive-coded macroblocks
    int recon_right_for_prev;
    int recon_down_for_prev;
    int recon_right_back_prev;
    int recon_down_back_prev;

    //int right_for, down_for, right_half_for, down_half_for;
    //int right_back, down_back, right_half_back, down_half_back;

    int prev_option;

    int *IDCT_Source;

public:

  //Variables to store picture (current, future, past)
    int* picture[3];   //(Y,Cb,Cr) each element is 10 bits
    int  seq_num[3];   //sequence number associated with picture[]

  //Variables to indicate which picture storage (0..2) is 
  //  back, future, current
  //  (ex) past_picture=n means picture[n] is past ref picture.
    int past_picture;     //the value is between 0 to 2
    int future_picture;   //the value is between 0 to 2
    int current_picture;  //the value is between 0 to 2

  //Error output file pointer
    FILE* err;

    MpegClass::MpegClass(FILE* in);

    MpegClass::~MpegClass(void){
        if(picture[0]!=NULL) delete picture[0];
        if(picture[1]!=NULL) delete picture[1];
        if(picture[2]!=NULL) delete picture[2];
    }

    //return 1 if decode correctly, 0 if end of sequence.
    int decode(void);

private:
    void get_sequence_header(void);
    void get_extension_user_data(void);
    void slice(void);
    void macroblock(void);
    void intra_macroblock(void);
    void deal_p_skipped_macroblock(int mb_addr_to_fill);
    void p_macroblock_in_p_picture(void);
    void deal_b_skipped_macroblock(int mb_addr_to_fill);
    void p_macroblock_in_b_picture(void);
    void ScaleIDCT(int* matrix);

    void output(int option, int mb_addr_to_fill);
            //option=0 : output without motion (for intra macroblock)
            //option=1 : with forward motion vector, and errors
            //option=2 : with backward motion vector only, and errors
            //option=3 : with both motion vectors, and errors
            //option=4 : with motion vector 0,0 , and errors
            //option=1+8 : with forward motion vector, no errors
            //option=2+8 : with backward motion vector only, no errors
            //option=3+8 : with both motion vectors, no errors
            //option=4+8 : with motion vector 0,0 , no errors

    inline void compute_motion(int rr, int rd, int x, int y,
                               int* pel_new, int* pel_ref);

    inline void compute_motion_temp(int rr, int rd, int x, int y,
                                    int* pel_new, int* pel_ref);
    
    inline void compute_both_motion(int x, int y);
    //inline void get_l_motion_vector_for(void);
    //inline void get_c_motion_vector_for(void);
    //inline void get_l_motion_vector_back(void);
    //inline void get_c_motion_vector_back(void);

    void test_macroblock(void);
    void test_block(int i);

};